#-*- coding: utf-8 -*-
"""
$Id$
"""

from zope.interface import implements

from zsqlmap import interfaces
from zsqlmap.utils import mixedToUnder
from zsqlmap.manager import ORMManager, get_orm 


__clsregister = {} # словарь всех SQLObject-классов. ключом является имя класса

def reg_class(name, cls):
    """регистрирует класс под его именем
    в clsregister. Если такое имя уже было,то кидается
    исключение.
    """
    __clsregister[name] = cls
    
def get_class(name):
    return __clsregister[name]


class _Base(object):
    """
    """
    implements(interfaces.ISQLObject)

    # ISQLObject
    select_sql = None
    table = None
    columns = () 
    _defcols = {} # map name to definition {'title': StringCol()}
    
    def __init__(self, id=None, **kw):        
        if id is None:
            self._create(**kw)
        else:
            self.id = id
            self._load(**kw)

    def _create(self, **kw):
        
        vals = {}
        for col in self.columns:
            if interfaces.IForeignKey.providedBy(col):
                k = col.name[:-3]
            else:
                k = col.name

            if kw.has_key(k):                
                v = kw[k]
            else:
                v = col.default
            
            vals[col.name] = col(v)                          
                
        self._vals = vals
        
        #print '\n DD', vals, self.columns
        self.id = get_orm().insertObject(self)

        
    def _load(self, **kw):
        vals = {}
        for col in self.columns:            
            k = col.name
            v = kw[k]
            if interfaces.IForeignKey.providedBy(col):
                if v is None:
                    vals[col.name] = None
                else:
                    vals[col.name] = int(v)
            else:
                vals[col.name] = col(v)                

        self._vals = vals

    
    def _get_fields(self):
        res = {}
        for col in self.columns: 
            res[col.name] = self._vals[col.name]
        return res

    fields = property(_get_fields)
    
    def _reg_dirty(self):
        get_orm().reg_dirty(self)


    def update_info(self):
        sql = "UPDATE %s SET %s WHERE id = %s"
        t = []
        params = []
        for k,v in self.fields.items():
            t.append('%s = %s' % (k,'%s'))
            params.append(v)

        sql = sql % (self.table, ','.join(t), self.id )
        return sql, tuple(params)

    def delete(self):
        get_orm().deleteObject(self)

    def _v_set_simple_property(self, name, value):
        self._vals[name] =  self._defcols[name](value)
        self._reg_dirty()

    def _v_get_simple_property(self, name):
        return self._vals[name]

    def _v_get_ref_property(self, name):
        fk = self._defcols[name]
        fk_id = self._vals[name]
        if fk_id is None:
            return None
        cls = get_class(fk.other_class)
        return cls.get(fk_id)

    def _v_get_multiple_joins(self, other_class, orderBy):
        cls = get_class(other_class)
        kw = {'%s_id' % self.table: self.id}
        return cls.selectBy(orderBy=orderBy,**kw)
        
    @classmethod
    def select(cls, condexpr=None, ignore_cache=False, orderBy=None):
        return get_orm().select(cls, condexpr, ignore_cache, orderBy)

    @classmethod
    def selectBy(cls, ignore_cache=False, orderBy=None, **kw):
        return get_orm().selectBy(cls, ignore_cache, orderBy, **kw )

    @classmethod
    def get(cls, id, ignore_cache=False):
        return get_orm().get(cls, id, ignore_cache)


    
class MT(type):

    def __new__(cls, name, bases, dict):
        columns = []
        defcols = {}
        cnames = []
        base_table = None
        #TODO: refactor (допустимо только одиночное наследование ? )
        for b in bases:
            if issubclass(b, SQLObject):
                if b.columns:
                    base_table = b.table
                    columns += b.columns
                    defcols = b._defcols.copy()
                    cnames += b._cnames
                    break
            
        for k,v in dict.items():
            
            if interfaces.IForeignKey.providedBy(v):
                v.name = '%s_id' % k
                cnames.append(v.name)
                columns.append(v)
                defcols[v.name] = v
                dict['_get_%s' % v.name] = eval("""lambda self: self._v_get_ref_property('%s')""" % v.name )
                dict['_set_%s' % v.name] = eval("""lambda self, value: self._v_set_simple_property('%s', value)  """ % v.name )
                dict[k] = property(dict['_get_%s' % v.name] , dict['_set_%s' % v.name])                

            elif interfaces.ICol.providedBy(v):
                v.name = k
                cnames.append(k)
                columns.append(v)
                defcols[k] = v
                dict['_get_%s' % k] = eval("""lambda self: self._v_get_simple_property('%s')""" % k )
                dict['_set_%s' % k] = eval("""lambda self, value: self._v_set_simple_property('%s', value)  """ % k )
                dict[k] = property(dict['_get_%s' % k] , dict['_set_%s' % k])

            elif interfaces.IMultipleJoin.providedBy(v):
                dict['_get_%s' % k] = eval("""lambda self: self._v_get_multiple_joins('%s','%s')""" % (v.other_class, v.orderBy or '') )
                dict[k] = property(dict['_get_%s' % k]) 

            
        dict['columns'] = tuple(columns)
        dict['_defcols'] = defcols
        dict['_cnames'] = cnames
        
        table = dict.get('table', None)
        if not table:
            table = name[0].lower() + mixedToUnder(name[1:])
            dict['table'] = table         
            
        select_sql = "SELECT id, %s FROM %s" % (','.join(cnames), table)
        dict['select_sql'] = select_sql

        if not bases:
            bases = (_Base,)
            
        o =  type.__new__(cls,name, bases, dict)

        reg_class(name, o)
        return o


class SQLObject:    
    __metaclass__ = MT


