#-*- coding: utf-8 -*-
"""
$Id$
"""
from decimal import Decimal
from datetime import date, datetime
from zope.interface import implements
from zsqlmap import interfaces

class DefaultError(Exception):
    pass

class NotNullError(Exception):
    pass

class ValidateError(Exception):
    pass


class Col(object):
    implements(interfaces.ICol)
    
    def __init__(self, **kw):
        self.name = None
        self.unique = kw.get('unique', False)
        self.notNull = kw.get('notNull', False)
        self._has_default =  kw.has_key('default') 

        if self._has_default:
            self._default = kw['default']
                
    def _get_default(self):
        if not self._has_default:
            raise DefaultError('%s %s' % (self,self.name))
        return self._default
    
    default = property(_get_default)
                
    def __call__(self, value):
        if value is None:
            if self.notNull:
                raise NotNullError(" column %s must be not None" % self.name)
        else:
            try:
                v = self._convert(value)
            except:
                raise ValidateError('%s bad format: %s' % (self, value) )
            return v
  
    
class StringCol(Col):
    implements(interfaces.IStringCol)
    
    def __init__(self, **kw):
        self.length = kw.pop('length', 255)
        super(StringCol, self).__init__(**kw)      

    def _convert(self, value):
        v = value.encode('ascii')
        if len(v) > self.length:
            raise ValidateError()
        return v

        
class UnicodeCol(StringCol):
    implements(interfaces.IUnicodeCol)
    
    def _convert(self, value):
        if isinstance(value, unicode):
            v = value
        else:
            v = unicode(value, 'utf-8')
            
        if len(v) > self.length:
            raise NotImplementedError()
        return v

class TextCol(Col):
    implements(interfaces.ITextCol)
    def _convert(self, value):
        if isinstance(value, unicode):
            v = value
        else:
            v = unicode(value, 'utf-8')
        return v
    

class IntCol(Col):
    implements(interfaces.IIntCol)
    def _convert(self, value):
        return int(value)

class BoolCol(Col):
    implements(interfaces.IBoolCol)
    def _convert(self, value):
        return bool(value)
    
class DecimalCol(Col):
    implements(interfaces.IDecimalCol)
    def __init__(self, **kw):
        self.size = kw.pop('size', 10)
        self.precision = kw.pop('precision', 4)
        super(DecimalCol, self).__init__(**kw)

    def _convert(self, value):
        return Decimal(str(value))


class CurrencyCol(Col):
    implements(interfaces.ICurrencyCol)

    size = 10
    precision = 2
    
    def _convert(self, value):
        res = Decimal(str(value))
        q = Decimal((0,(1,),-2))    
        res = res.quantize(q)    
        return res

class DateTimeCol(Col):
    implements(interfaces.IDateTimeCol)
    def _convert(self, value):
        if isinstance(value, datetime):
            return value
        elif isinstance(value, date):
            return datetime(value.year, value.moth, value.day)

        raise ValidateError('bad format: %s' % value)


class DateCol(Col):
    implements(interfaces.IDateCol)
    def _convert(self, value):
        if isinstance(value, datetime):
            return date(value.year, value.month, value.day)
        elif isinstance(value, date):
            return value
        
        raise ValidateError('bad format: %s' % value)


class ForeignKey(Col):
    implements(interfaces.IForeignKey)
    def __init__(self, other_class, **kw):
        self.other_class = other_class
        super(ForeignKey, self).__init__(**kw)
    
    def _convert(self, value):
        return value.id

     
class MultipleJoin:
    implements(interfaces.IMultipleJoin)
    def __init__(self, other_class, orderBy=None):
        self.other_class = other_class
        self.orderBy = orderBy
