#-*- coding: utf-8 -*-
"""
$Id$
"""
from zope.interface import Interface, Attribute
from zope.schema import TextLine

class IORMManager(Interface):
    """
    """
    da_name = TextLine(title=u'DA name',
                       default=u'',
                       description=u"""все операции с БД будут
                                      делаться через этот адаптер""")

    connection = Attribute('read-only: IZopeConnection for current thread')

    def configure_for_request():
        """ ОБЯЗАТЕЛЬНО вызывать перед началом обработки каждого
        запроса.
        """

    def clear_cache():
        "cleare cache"
        
    def insertObject(obj):
        """ делает insert объекта (ISQLObject) в БД
        возвращает его идентификатор.
        """

    def deleteObject(obj):
        """ удаляет объект из базы
        """
        
    def get(cls, id, ignore_cache=False):
        """ возвращает объект заданного класса по его id.
        Внутри потока каждый объект, определяемый парой (cls,id) ,
        должен существовать в единтсвенном экземпляре.
        По умолчанию поиск объекта сперва производится в кеше.
        """
    def select(cls, condexpr, ignore_cache=False, orderBy=None):
        "return IResult "
        
    def selectBy(cls, ignore_cache=False, orderBy=None, **kw):
        "return IResult"
        
    def reg_dirty(obj):
        """ добавляет obj в список изменённых объектов.
        Когда транзакция будет коммитится, то все изменённые объекты будут
        скинуты в базу (сделан UPDATE).
        При откате\коммите список будет очищен.
        Список локален для каждого потока.
        """

    def createTableSQL(cls):
        """ возвращает последовательность(как итератор) SQL-команд,
        которые необходимы для создания таблицы на основе SQLObject-класса
        """

    def createTable(cls):
        """ создаёт таблицу в БД на основе SQLObject-класса
        """
        
    def commit():
        """ делает UPDATE для изменённых объектов, чистит кеш и список изменённых объектов.
        напрямую не вызывать, только через transaction.commit()
        """
        
    def rollback():
        """ чистит кеш и список изменённых объектов.
        напрямую не вызывать, только через transaction.abort()
        """

    def queryAddLimitOffset(query, start, end):
        """ добавляет в запрос ограничения на выборку.
        предполагается что хотя бы один из (start, end)
        не None.
        return query with limits"""

    def addColumnSQL(cls, col):
        """ возвращает SQL-dвыражение, которое
        необходимо выполнить для добавления
        нового столбца в таблицу, соответствующую cls.
        """

class IConditionExpr(Interface):
    sql = Attribute('sql expression')
    params = Attribute('tuple of params')
    
class IConditionManager(Interface):
    def AND(*condexprs):
        """ condexprs - list of IConditionExpr
            return IConditionExpr
        """
    
    def OR(*condexprs):
        """ condexprs - list of IConditionExpr
            return IConditionExpr
        """

    def NOT(condexpr):
        """ condexpr - IConditionExpr
            return IConditionExpr
        """

    def EQ(name, value):
        """ ==
        return IConditionExpr
        """

    def NE(name, value):
        """ !=
        return IConditionExpr
        """

    def LT(name, value):
        """ <
        return IConditionExpr
        """
        
    def LE(name, value):
        """ <=
        return IConditionExpr
        """

    def GT(name, value):
        """ >
        return IConditionExpr
        """
        
    def GE(name, value):
        """ >=
        return IConditionExpr
        """
    
    def LIKE(name, value):
        """ return IConditionExpr
        """

    def ILIKE(name, value):
        """ return IConditionExpr
        """
        
    def BETWEEN(name, r1, r2):
        """ return IConditionExpr
        """

    def ISNULL(name):
        """ return IConditionExpr
        """



    
class ISQLObject(Interface):
    id = Attribute('id')
    select_sql = Attribute('sql: select id,... from table')

    table = Attribute('наименование таблицы')
    columns = Attribute("""список(tuple) столбцов (ICol) (за исключением id),
                           в том порядке в котором эти имена используются в select, insert, update""")

    fields = Attribute('словарь {column_name:value} без id')

    def update_info():
        """ возвращает пару sql_update, params. используется для
        записи изменений объекта в БД
        sql_update не должен содержать условие WHERE
        """
    def select(condexpr, ignore_cache=False, orderBy=None):
        "return IResult "

    def selectBy(ignore_cache=False, orderBy=None, **kw):
        "return IResult"
        
    def get(id, ignore_cache=False):
        """ Внутри потока каждый объект, определяемый парой (cls,id) ,
        должен существовать в единственном экземпляре.
        По умолчанию поиск объекта сперва производится в кеше.
        """
        
    def delete():
        "удаляет запись об объекте из базы"
    
class IMultipleJoin(Interface):
    "описывает join"
    name = Attribute('name of property')
    other_class = Attribute('name of other class')
    orderBy = Attribute('order by string')


class ICol(Interface):
    name = Attribute('name of column')
    default = Attribute('default value')
    unique = Attribute('True|False')
    notNull = Attribute('True|False')

    
    def __call__(value):
       "convert"
    
class IStringCol(ICol):
    length = Attribute('integer')

class IUnicodeCol(IStringCol):
    pass

class ITextCol(ICol):
    pass

class IIntCol(ICol):
    pass

class IBoolCol(ICol):
    pass

class IDecimalCol(ICol):
    size = Attribute('')
    precision = Attribute('')

class ICurrencyCol(IDecimalCol):
    pass

class IDateCol(ICol):
    pass

class IDateTimeCol(ICol):
    pass


class IForeignKey(ICol):
    other_class = Attribute('name of other class')




class IResult(Interface):
    """ интерфейс для результата выборки
    """
    
    def __len__():
        "возвращает кол-во строк в выборке"

    def __iter__():
        """ возвращает итератор по строкам в выборке
        """
        
    def __getitem__(index):
        "возвращает срез"

    def __getitem__(index):
        "возвращает элемент по индексу"

    def __getslice__(i,j):
        """возвращает срез как новый IResult
        у полученного IResult методы count и т.п.
        будут выдавать не верные результаты (TODO?)        
        """
        
    def aggregate(expr):
        """ делает агрегирующий запрос expr
        на выборке        
        """
