#-*- coding: utf-8 -*-
"""
$Id$
"""
from persistent import Persistent
import transaction
from transaction.interfaces import IDataManager
from zope.interface import implements
from zope.app import zapi
from zope.app.container.contained import Contained
from zope.rdb.interfaces import IZopeDatabaseAdapter 
from threading import local

from interfaces import IORMManager, IConditionExpr, IResult
from cond import ConditionManager
from result import Result
thread_data = local()
# используются следующие локальные переменные:
# orm_manager - IORMManager
# connection - IZopeConnection 
# dirtys - словарь изменённых объектов. очищается при каждом коммите\откате
# cache - словарь считанных из базы объектов для кеширования. очищается при каждом коммите\откате
# txn_registered - True если участвуем в транзакции, иначе False

def get_orm():
    return thread_data.orm_manager

class ORMManager(Persistent, Contained, ConditionManager):
    implements(IORMManager)

    result_factory = Result
    
    def __init__(self, da_name=''):
        self.da_name = da_name
        
    connection = property(lambda _: thread_data.connection)
    
    def configure_for_request(self):
        da = zapi.getUtility(IZopeDatabaseAdapter, name=self.da_name, context=self.__parent__)
        thread_data.connection = da()
        thread_data.orm_manager = self      
        thread_data.dirtys = {}        
        thread_data.cache = {}
        thread_data.txn_registered = False

    def clear_cache(self):
        thread_data.cache.clear()
        
    def restyle(self, sql):
        """по умолчанию используется paramstyle == 'format' (т.е. %s).
        переопределение данного метода позволит изменить это.
        например:
           return sql.replace('%s', '?')
        """
        return sql
    
    def _registerForTxn(self):
        " подключаемся к текущей транзакции (если ещё не подключены)"

        self.connection.registerForTxn() # на всякий случай

        if not thread_data.txn_registered:
            tm = ORMTransactionManager(self)
            transaction.get().join(tm) 
            thread_data.txn_registered = True
        
    def _to_cache(self, obj, id): 
        cache_key = '%s_%s' % (obj.table, id)
        thread_data.cache[cache_key] = obj
        
    def _from_cache(self, cls, id):
        cache_key = '%s_%s' % (cls.table, id)        
        obj = thread_data.cache.get(cache_key, None)
        return obj

    def _clear(self):
        thread_data.dirtys.clear()
        thread_data.cache.clear()
        
    def get(self, cls, id, ignore_cache=False):
        
        if not ignore_cache:
            # сперва ищем в кеше
            obj = self._from_cache(cls, id)
            if not (obj is None):
                return obj
        
        # если в кеше нет или он игнорирован, то загружаем из базы 
        sql = "%s WHERE id = %s" % (cls.select_sql, int(id) )
        cur = self.connection.cursor()
        cur.execute(sql)
        row = cur.fetchone()
        
        if row is None:
            return None

        kw = {}
        i = 1 # считаем что id есть в SELECT'е
        for c in cls.columns:
            kw[c.name] = row[i]
            i += 1
            
        return self._make_object(cls, id, **kw)

    def _make_object(self, cls, id, **kw):
        obj = cls(id=id, **kw)        
        # кладём в кеш
        self._to_cache(obj, id)        
        return obj             
    
    def select(self, cls, condexpr=None, ignore_cache=False, orderBy=None):
        params = ()
        cond = ''
        if IConditionExpr.providedBy(condexpr):
            cond = "WHERE %s" %  condexpr.sql            
            params = condexpr.params            
        elif isinstance(condexpr, str):
            cond = "WHERE %s" % condexpr            
        elif condexpr:
            raise Exception(" bad conditions %s" % condexpr)

        sql_orderby = (orderBy and ' ORDER BY %s ' % orderBy) or ''

        res = self.result_factory(self, cls, cond, params, sql_orderby, ignore_cache)
        return res
    
        
    def selectBy(self, cls, ignore_cache=False, orderBy=None, **kw):
        expr = []
        for k,v in kw.items():
            if v is None:
                expr.append(self.ISNULL(k))
            else:
                expr.append(self.EQ(k,v))
        return self.select(cls, self.AND(*expr), ignore_cache=ignore_cache, orderBy=orderBy)
                 
    def reg_dirty(self, obj):
        # раз есть изменённый объект
        # то значит это надо делать в транзакии
        self._registerForTxn()
        thread_data.dirtys[id(obj)] = obj

    def deleteObject(self, obj):
        self._registerForTxn()
        sql = "DELETE FROM %s WHERE id = %s" % (obj.table, obj.id)        
        cur = self.connection.cursor()
        cur.execute(sql)


    def createTable(self, cls):
        self._registerForTxn() 
        
        cmds = self.createTableSQL(cls)
        cur = self.connection.cursor()
        try:
            for cmd in cmds:
                cur.execute(cmd)
        except:
            pass
        
    def commit(self):
        """ 
        """
        thread_data.txn_registered = False        
        cur = self.connection.cursor()
        for obj in thread_data.dirtys.values():
            sql, params = obj.update_info()
            cur.execute(self.restyle(sql), params)

        self._clear() # очищаемся

        
    def rollback(self):
        """
        """
        thread_data.txn_registered = False
        self._clear()  # очищаемся
        
    
    def insertObject(self, obj):
        self._registerForTxn()
        
        new_id = self._generateID(obj)
        cols = ['id']
        params = [new_id]
        t = ['%s']
        for k, v in obj.fields.items():
            cols.append(k)
            params.append(v)
            t.append('%s')
       
        sql = "INSERT INTO %s (%s) VALUES (%s)" % (obj.table,
                                                  ','.join(cols),
                                                  ','.join(t)
                                                  )
         
        cur = self.connection.cursor()       
        
        cur.execute(self.restyle(sql), tuple(params))
        
        # to cache
        self._to_cache(obj, new_id)
        
        return new_id


    # must be overwrite in childrens class
    def _generateID(self, obj):
        raise NotImplementedError

    def createTableSQL(self, cls):
        raise NotImplementedError

    def queryAddLimitOffset(self, query, start, end):
        raise NotImplementedError

    def addColumnSQL(self, cls, col):
        raise NotImplementedError

    
class ORMTransactionManager(object):

    implements(IDataManager)

    def __init__(self, orm):
        self._orm = orm
        self.transaction_manager = transaction.manager

    
    def tpc_begin(self, txn):
        """ когда транзакцию начинают коммитить,
        то мы закоммитимся наперёд
        """
        self._orm.commit()
        
    def tpc_vote(self, txn):
        pass

    def tpc_finish(self, txn):
        pass

    def tpc_abort(self, txn):
        pass

    def abort(self, txn):
        self._orm.rollback()
        
    def commit(self, txn):
        pass

    def sortKey(self):
        "очень хотим коммитится впереди коммита DA"
        return 'aazsqlmap' + str(id(self))


    
