#-*- coding:utf-8 -*-
"""  ORM Manager for Postgres
$Id$
"""
from zsqlmap import ORMManager
from zsqlmap import interfaces
from zsqlmap.cond import CondExpr
from zsqlmap.result import StubSliceResult

class ORM_MSSQL(ORMManager):

    result_factory = StubSliceResult
    
    ifaces = [interfaces.IStringCol ,
              interfaces.ITextCol,
              interfaces.IIntCol ,interfaces.IBoolCol ,
              interfaces.IDecimalCol,
              interfaces.IDateCol ,interfaces.IDateTimeCol ,
              interfaces.IForeignKey
              ]


    def insertObject(self, obj):
        self._registerForTxn()

        cols = []
        params = []
        t = []
        for k, v in obj.fields.items():
            cols.append(k)
            params.append(v)
            t.append('%s')
       
        sql = "INSERT INTO %s (%s) VALUES (%s)" % (obj.table,
                                                  ','.join(cols),
                                                  ','.join(t)
                                                  )
         
        cur = self.connection.cursor()       
               
        cur.execute(self.restyle(sql), tuple(params))
               
        # read new_id
        cur.execute("SELECT @@IDENTITY")
        new_id = int(cur.fetchone()[0])
        
        # to cache
        self._to_cache(obj, new_id)
        
        return new_id


    def createTable(self, cls):
        if not self._checkTable(cls):
            return
        
        self._registerForTxn()
        cols_defs = ['id integer IDENTITY PRIMARY KEY']
        for col in cls.columns:
            cols_defs.append(self._column_sql(col))
            
        
        sql = "CREATE TABLE %s (%s)" % (cls.table, ','.join(cols_defs))
        
        cur = self.connection.cursor()
        cur.execute(sql)

        
    def _checkTable(self, cls):
        sql = "SELECT 1 FROM sysobjects WHERE xtype='U' AND name='%s'" % cls.table
        cur = self.connection.cursor()
        cur.execute(sql)
        rows = cur.fetchall()
        if len(rows) == 0:
            return True
                

    def createTableSQL(self, cls):
        yield "CREATE TABLE %s (id integer IDENTITY PRIMARY KEY)" % cls.table
        for col in cls.columns:
            s = self.addColumnSQL(cls, col)
            yield s

        
    def addColumnSQL(self, cls, col):
        return "ALTER TABLE %s ADD %s" % (cls.table, self._column_sql(col) ) 
        
    def _column_sql(self, col):
        for iface in self.ifaces:
            if iface.providedBy(col):
                f = getattr(self, 'sql_%s' % iface.__name__[1:], None)
                if f is None:
                    raise NotImplementedError("This ORM not support '%s' type" % iface.__name__)
                sql = f(col)
                if col.notNull:
                    sql += " NOT NULL "
                if col.unique:
                    sql += " UNIQUE "
                return sql
    
    def sql_StringCol(self, col):
        sql = "%s VARCHAR(%i)" % (col.name, col.length)
        return sql

    def sql_TextCol(self, col):
        sql = "%s TEXT" % col.name
        return sql

    def sql_IntCol(self, col):
        sql = "%s INTEGER " % col.name
        return sql

    def sql_BoolCol(self, col):
        sql = "%s BIT " % col.name
        return sql
    
    def sql_DecimalCol(self, col):
        sql = "%s DECIMAL(%i, %i)" %(col.name, col.size, col.precision)
        return sql

    def sql_DateCol(self, col):
        sql = "%s DATETIME" % col.name
        return sql
    
    def sql_DateTimeCol(self, col):
        sql = "%s DATETIME" % col.name
        return sql

    
    def sql_ForeignKey(self, col):
        sql = "%s INTEGER" % col.name
        return sql

    def ILIKE(self, name, value):
        return CondExpr('UPPER(%s) LIKE %s' % (name,'%s'), (value.upper(),))

    def ISNULL(self, name):
        return CondExpr('%s IS NULL' % name, ())

    def queryAddLimitOffset(self, query, start, end):
        return query
##         if not start:
##             return "%s LIMIT %i" % (query, end)
##         if not end:
##             return "%s OFFSET %i" % (query, start)
##         return "%s LIMIT %i OFFSET %i" % (query, end-start, start)
