#-*- coding:utf-8 -*-
"""  ORM Manager for Postgres
$Id$
"""
from zsqlmap import ORMManager
from zsqlmap import interfaces
from zsqlmap.cond import CondExpr

class ORMPostgres(ORMManager):

    ifaces = [interfaces.IStringCol ,
              interfaces.ITextCol,
              interfaces.IIntCol ,interfaces.IBoolCol ,
              interfaces.IDecimalCol,
              interfaces.IDateCol ,interfaces.IDateTimeCol ,
              interfaces.IForeignKey
              ]
    
    def _generateID(self, obj):
        sql = "SELECT NEXTVAL ('seq_pkey_%s')" % obj.table        
        cur = self.connection.cursor()
        cur.execute(sql)
        nid = cur.fetchone()[0]
        return nid

    def createTableSQL(self, cls):
        yield "CREATE SEQUENCE seq_pkey_%s" % cls.table
        yield "CREATE TABLE %s (id integer PRIMARY KEY)" % cls.table
        
        for col in cls.columns:            
            yield self.addColumnSQL(cls, col)

    def addColumnSQL(self, cls, col):
        return "ALTER TABLE %s ADD COLUMN %s" % (cls.table, self._column_sql(col) ) 
        
    def _column_sql(self, col):
        for iface in self.ifaces:
            if iface.providedBy(col):
                f = getattr(self, 'sql_%s' % iface.__name__[1:], None)
                if f is None:
                    raise NotImplementedError("This ORM not support '%s' type" % iface.__name__)
                sql = f(col)
                if col.notNull:
                    sql += " NOT NULL "
                if col.unique:
                    sql += " UNIQUE "
                return sql
    
    def sql_StringCol(self, col):
        sql = "%s VARCHAR(%i)" % (col.name, col.length)
        return sql

    def sql_TextCol(self, col):
        sql = "%s TEXT" % col.name
        return sql

    def sql_IntCol(self, col):
        sql = "%s INTEGER " % col.name
        return sql

    def sql_BoolCol(self, col):
        sql = "%s BOOL" % col.name
        return sql
    
    def sql_DecimalCol(self, col):
        sql = "%s DECIMAL(%i, %i)" %(col.name, col.size, col.precision)
        return sql

    def sql_DateCol(self, col):
        sql = "%s DATE" % col.name
        return sql
    
    def sql_DateTimeCol(self, col):
        sql = "%s TIMESTAMP" % col.name
        return sql

    
    def sql_ForeignKey(self, col):
        sql = "%s INTEGER" % col.name
        return sql

    def ILIKE(self, name, value):
        """  нормальную поддержку ILIKE обещают сделать в PostgreSQL 8.2
        поэтому пользуемся приведением к UPPERCASE.
        Работает только на ru_RU.UTF8.
        """
        return CondExpr('UPPER(%s) LIKE %s' % (name,'%s'), (value.upper(),))


    def queryAddLimitOffset(self, query, start, end):
        if not start:
            return "%s LIMIT %i" % (query, end)
        if not end:
            return "%s OFFSET %i" % (query, start)
        return "%s LIMIT %i OFFSET %i" % (query, end-start, start)
