#-*- coding: utf-8 -*-
"""
$Id$
"""
from zope.interface import implements
from interfaces import IResult

class Result:
    "реализация IResult для СУБД которые поддерживают срезы (OFFSET & LIMIT) на уровне SQL"
    implements(IResult)

    native_slice = True
    
    def __init__(self, orm, cls, cond, params, sql_orderby, ignore_cache, start=None, end=None):
        self.orm = orm
        self.cls = cls
        self.cond = cond
        self.params = params
        self.sql_orderby = sql_orderby
        self.ignore_cache = ignore_cache
        self.cur = orm.connection.cursor()
        self.start = start
        self.end = end 

    def finalPrepareQuery(self, query):
        "final prepare"
        if self.start or self.end:
            sql = self.orm.queryAddLimitOffset(query, self.start, self.end)
            if sql == query:
                raise NotImplementedError('slice not supported')
            query = sql
            
        query = self.orm.restyle(query)
        return query

    
    def __iter__(self):
        cur = self.cur
        orm = self.orm
        cls = self.cls
        
        if self.ignore_cache:
            sql =  '%s %s %s' % (cls.select_sql, self.cond, self.sql_orderby)
            sql = self.finalPrepareQuery(sql)
            cur.execute(sql, self.params)
            rows = cur.fetchall()
            
            for r in rows:
                kw = {}
                id = r[0]
                i = 1 # считаем что id есть в SELECT'е
                for c in cls.columns:
                    kw[c.name] = r[i]
                    i += 1
                yield cls(id=id, **kw)
            
        else:
            sql = ("SELECT id FROM %s " % self.cls.table) + self.cond + self.sql_orderby
            sql = self.finalPrepareQuery(sql)
            cur.execute(sql, self.params)
            rows = cur.fetchall()        
            for r in rows:
                yield orm.get(self.cls, r[0])

    def __len__(self):
        return int(self.aggregate('COUNT(*)') or 0)

    def __getitem__(self, index):
        r = self.__getslice__(index,index+1)
        for obj in r:
            return obj
        
    def __getslice__(self, i,j):
        return Result(self.orm, self.cls, self.cond,
                      self.params, self.sql_orderby, self.ignore_cache, i, j)
    
    def aggregate(self, expr):
        sql = ("SELECT %s FROM %s " % (expr, self.cls.table) ) + self.cond
        sql = self.finalPrepareQuery(sql)
        self.cur.execute(sql, self.params)
        rows = self.cur.fetchall()
        return rows and rows[0][0] or None
 


class StubSliceResult(Result):
    native_slice = False

    def finalPrepareQuery(self, query):
        "final prepare"
        query = self.orm.restyle(query)
        return query

    def check_pos(self, pos):
        """ определяет попадает объект в рамки self.start & или нет.
        выдаёт:
        True - если попадает
        False - не попадает
        None - уже произошёл выход за рамки
        """
        if not (self.start or self.end):
            return True
        
        if not self.start:
            if pos >= self.end:
                return None
            else:
                return True

        if not self.end:
            if pos >= self.start:
                return True
            else:
                return False

        if pos < self.start:
            return False
        if pos >= self.end:
            return None
        return True


            
    def __iter__(self):
        cur = self.cur
        orm = self.orm
        cls = self.cls

        if self.ignore_cache:
            sql =  '%s %s %s' % (cls.select_sql, self.cond, self.sql_orderby)
            sql = self.finalPrepareQuery(sql)
            cur.execute(sql, self.params)
            rows = cur.fetchall()

            pos = 0
            for r in rows:
                kw = {}
                id = r[0]
                i = 1 # считаем что id есть в SELECT'е
                for c in cls.columns:
                    kw[c.name] = r[i]
                    i += 1

                chk = self.check_pos(pos)
                if chk:
                    yield cls(id=id, **kw)
                elif chk is None:
                    break
                pos +=1
                
        else:
            sql = ("SELECT id FROM %s " % self.cls.table) + self.cond + self.sql_orderby
            sql = self.finalPrepareQuery(sql)
            cur.execute(sql, self.params)
            rows = cur.fetchall()
            pos = 0
            for r in rows:
                chk = self.check_pos(pos)
                if chk:
                    yield orm.get(self.cls, r[0])
                elif chk is None:
                    break
                pos +=1

    def __getslice__(self, i,j):
        return StubSliceResult(self.orm, self.cls, self.cond,
                               self.params, self.sql_orderby, self.ignore_cache, i, j)

