#-*- coding:utf-8 -*-
""" 
$Id$
"""
from decimal import Decimal
from datetime import date, datetime
import unittest

from zope.interface import implements
from zope.interface.verify import verifyObject, verifyClass
from zsqlmap import SQLObject
from zsqlmap.cols import *
from zsqlmap import interfaces

class TestColumn(unittest.TestCase):

    def test_string_column(self):
        dc = StringCol()
        self.assert_(verifyClass(interfaces.IStringCol, StringCol))
        self.assertEqual( dc.unique, False)
        self.assertEqual( dc.length, 255)
        
        self.assertRaises( DefaultError, lambda : dc.default)
        self.assertRaises( ValidateError, lambda : dc('сывц'))
        
        self.assertEqual( dc('qwert'), 'qwert')

        dc = StringCol(default='abc', unique=True, length=5)

        self.assertEqual(dc.default, 'abc')
        self.assertEqual( dc.unique, True)
        self.assertRaises( ValidateError, lambda : dc('123456789'))
        self.assertEqual( dc(None), None)
        
        # check notNull
        dc = StringCol(notNull=True)
        self.assertRaises( NotNullError, lambda : dc(None))
        
        
    def test_unicode_column(self):
        dc = UnicodeCol()
        self.assert_(verifyClass(interfaces.IUnicodeCol, UnicodeCol ))
        self.assertEqual( dc.unique, False)
        self.assertEqual( dc.length, 255)
        
        self.assertRaises( DefaultError, lambda : dc.default)
        
        self.assertEqual( dc('qwert'), u'qwert')

        dc = UnicodeCol(default='abc', unique=True, length=5)

        self.assertEqual(dc.default, u'abc')
        self.assertEqual( dc.unique, True)
        self.assertRaises( ValidateError, lambda : dc('123456789'))
        
        
    def test_int_column(self):
        dc = IntCol()
        self.assert_(verifyClass(interfaces.IIntCol, IntCol))
        self.assertEqual( dc.unique, False)
        
        self.assertRaises( DefaultError, lambda : dc.default)
        self.assertRaises( ValidateError, lambda  :dc('abc'))
        
        self.assertEqual( dc(123), 123)
        self.assertEqual( dc(123.2356), 123)

        dc = IntCol(default=234, unique=True)

        self.assertEqual(dc.default, 234)
        self.assertEqual( dc.unique, True)

    def test_bool_column(self):
        dc = BoolCol()
        self.assert_(verifyClass(interfaces.IBoolCol, BoolCol))

        self.assertEqual( dc(1), True)
        self.assertEqual( dc(None), None)
        self.assertEqual( dc(0), False)

        
    def test_decimal_column(self):
        dc = DecimalCol()
        self.assert_(verifyClass(interfaces.IDecimalCol, DecimalCol))
        self.assertEqual( dc.unique, False)
        
        self.assertRaises( DefaultError, lambda : dc.default)
        self.assertRaises( ValidateError, lambda  :dc('abc'))

        self.assertEqual(dc('567.78'), Decimal('567.78'))

    def test_currency_column(self):
        dc = CurrencyCol()
        self.assert_(verifyClass(interfaces.ICurrencyCol, CurrencyCol))
        self.assertEqual( dc.unique, False)
        
        self.assertRaises( DefaultError, lambda : dc.default)
        self.assertRaises( ValidateError, lambda  :dc('abc'))

        self.assertEqual(dc('567.785999'), Decimal('567.79'))
        
    def test_datetime_col(self):
        dc = DateTimeCol()
        self.assert_(verifyClass(interfaces.IDateTimeCol, DateTimeCol))

        self.assertRaises( DefaultError, lambda : dc.default)
        self.assertRaises( ValidateError, lambda  :dc('abc'))
        
        now = datetime.now()
        self.assertEqual(dc(now), now)

    def test_date_col(self):
        dc = DateCol()
        self.assert_(verifyClass(interfaces.IDateCol, DateCol))

        self.assertRaises( DefaultError, lambda : dc.default)
        self.assertRaises( ValidateError, lambda  :dc('abc'))
        
        now = datetime.now()
        today = date.today()
        self.assertEqual(dc(now), today )
        
    def test_foreign_key(self):
        dc = ForeignKey('OtherClass')
        self.assert_(verifyClass(interfaces.IForeignKey, ForeignKey))
        self.assertEqual( dc.unique, False)
        
        self.assertRaises( DefaultError, lambda : dc.default)
        self.assertRaises( ValidateError, lambda  :dc('abc'))
        

    def test_declare(self):
        class MyTable(SQLObject):
            title = StringCol()
            amount = IntCol()

        self.assert_( verifyClass(interfaces.ISQLObject, MyTable) )

        # select_sql
        sql = MyTable.select_sql
        self.assert_('title' in sql)
        self.assert_('amount' in sql)
        
        # table
        self.assertEqual(MyTable.table, 'my_table')

        # columns
        col = filter(lambda x: x.name=='title', MyTable.columns)[0]
        #self.assert_( verifyObject(interfaces.IStringCol, col) )
        self.assertEqual(col.name, 'title')

        col = filter(lambda x: x.name=='amount', MyTable.columns)[0]
        #self.assert_( verifyObject(interfaces.IStringCol, col) )
        self.assertEqual(col.name, 'amount')
        
        # constructor validate        
        self.assertRaises( ValidateError, lambda : MyTable(title='ыыыы', amount=123) )
        self.assertRaises( ValidateError, lambda : MyTable(title='title', amount='123.676') )
        self.assertRaises( DefaultError, lambda : MyTable(amount=123) )

        
def test_suite():
    return unittest.TestSuite((
        unittest.makeSuite(TestColumn),
        ))

if __name__ == '__main__':
    unittest.main(defaultTest='test_suite')


